<?php
namespace Teknisa\CUS\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class Cus01400 extends \Zeedhi\Framework\Controller\Report { 

    protected $novoCodigo;
    protected $consplan;
    protected $dataSourceManager;
    protected $codigo;
    protected $environment;
    protected $entityManager;
    protected $birtStrategy;
    protected $reportService;

    public function __construct($crudService, $parameterBag, $consplan, $novoCodigo, $entityManager, $dataSourceManager, $environment, $birtStrategy, $reportService) {
        $this->crudService       = $crudService;
        $this->parameterBag      = $parameterBag;
        $this->consplan          = $consplan;
        $this->novoCodigo        = $novoCodigo;
        $this->entityManager     = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment       = $environment;
        $this->birtStrategy      = $birtStrategy;
        $this->reportService     = $reportService;
    }

    protected $reportMapping = array (
        'CUS01400' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => array (
                'CDTIPOCUSTO'      => 'P_NRTIPOCUSTO',
                'NMTIPOCUSTO'      => 'P_NMCUSTO',
                'IDMATEPRIM'       => 'P_IDMATEPRIM',
                'IDNESTOCAVEIS'    => 'P_IDNESTOCAVEIS',
                'IDCONSPERC'       => 'P_IDCONSPERC',   
                'IDCONSFIXO'       => 'P_IDCONSFIXO',
                'CDFILIAL'         => 'P_CDFILIAL',
                'NRCONSPLAN'       => 'P_NRCONSPLAN',
                'DTMESINIC'        => 'P_DTMESINIC',
                'DTMESFINA'        => 'P_DTMESFINA',
                'DTINICIAL'        => 'P_DTINICIAL',
                'DTFINAL'          => 'P_DTFINAL',
                'CDARVPRODINI'     => 'P_CDPRODINIC',
                'CDARVPRODFINAL'   => 'P_CDPRODFINA',
                'CDPICTPROD'       => 'P_MASK',
                'NRORG'            => 'P_NRORG'
            )
        ),
        'CUS01401' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => array (
                'CDTIPOCUSTO'      => 'P_NRTIPOCUSTO',
                'NMTIPOCUSTO'      => 'P_NMTIPOCUSTO',
                'CDFILIAL'         => 'P_CDFILIAL',
                'NRCONSPLAN'       => 'P_NRCONSPLAN',
                'DTMESINIC'        => 'P_DTMESINIC',
                'DTMESFINA'        => 'P_DTMESFINA',
                'IDNESTOCAVEIS'    => 'P_IDNESTOCAVEIS',
                'DTINICIAL'        => 'P_DTINICIAL',
                'DTFINAL'          => 'P_DTFINAL',
                'CDARVPRODINI'     => 'P_CDPRODINIC',
                'CDARVPRODFINAL'   => 'P_CDPRODFINA',
                'IDCONSFIXO'       => 'P_IDCONSFIXO',
                'IDCONSPERC'       => 'P_IDCONSPERC',   
                'IDMATEPRIM'       => 'P_IDMATEPRIM',
                'IDVISUALIZA'      => 'P_PRODZER',
                'CDPICTPROD'       => 'P_MASK',
                'NRORG'            => 'P_NRORG'
            )
        )
    );

    public function createRemoteReport(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $this->beforeProcessReport($request, $response);
            $reportURL = $this->processReport($request);
            $response->addMessage(new DTO\Response\Message($reportURL));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        } 
    }
    
     public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $row["NRORG"] = $this->environment->getNrOrg();
        $this->procedureConsumoPlanejado($row);
        
        if (isset($row['QR2TYPE']) && $row['QR2TYPE']){
          $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        }
    }

    public function procedureConsumoPlanejado($row) {
        $dataFinal   = \DateTime::createFromFormat('d/m/Y', $row['DTFINAL']);
        $dataInicial = \DateTime::createFromFormat('d/m/Y', $row['DTINICIAL']);

        $tamanho = 20;
        $completaCom = 'z';
        $row['CDPRODUTOFINAL'] = str_pad($row['CDPRODUTOFINAL'], $tamanho, $completaCom, STR_PAD_RIGHT);
        $row['CDARVPRODFINAL'] = str_pad($row['CDARVPRODFINAL'], $tamanho, $completaCom, STR_PAD_RIGHT);
        $codigo = $this->novoCodigo->getCodeByTableName('CONSPLAN'.$row['CDFILIAL'], $row['NRORG']);
        $connection = $this->entityManager->getConnection();
        $procedureConsumoPlanejado = new StoredProcedure($connection, 'CONSUMO_PLANEJADO_PLA');
        $procedureConsumoPlanejado->addParam(new Param('P_FILIAL',       Param::PARAM_INPUT, $row['CDFILIAL'],                    Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_SERVICOS',     Param::PARAM_INPUT, 'T',                                 Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $row['DTINICIAL'],                   Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_DTFINAL',      Param::PARAM_INPUT, $row['DTFINAL'],                     Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDARQUIVO',    Param::PARAM_INPUT, "T",                                 Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDPRODUTOS',   Param::PARAM_INPUT, $row['IDPRODUTOS'],                  Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_IDSEPPRATO',   Param::PARAM_INPUT, 'N',                                 Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_PEACRESCIMO',  Param::PARAM_INPUT, 0,                                   Param::PARAM_TYPE_INT));
        $procedureConsumoPlanejado->addParam(new Param('P_IDARREDOND',   Param::PARAM_INPUT, "N",                                 Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_NRCONSPLAN',   Param::PARAM_INPUT, $codigo,                             Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, $row['CDARVPRODINI'],                Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, $row['CDARVPRODFINAL'],              Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $row['CDOPERADOR'],                  Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_STDTINICIAL',  Param::PARAM_INPUT, "_".$dataInicial->format('Ymd')."_", Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->addParam(new Param('P_STDTFINAL',    Param::PARAM_INPUT, "_".$dataFinal->format('Ymd')."_",   Param::PARAM_TYPE_STR));
        $procedureConsumoPlanejado->execute();
        $this->params = array();

        $row['NRCONSPLAN']      = $codigo;
        $row['DTPERIINIC']      = $row['DTINICIAL'];
        $row['DTPERIFINA']      = $row['DTFINAL'];
        $row['CDTIPOCUSTO'] =  (int)$row['CDTIPOCUSTO'];
        $row['CDPRODINIC'] = $row['CDPRODUTOINI'];
        $row['CDPRODFINA'] = $row['CDPRODUTOFINAL'];

        $this->nrorg      = $row['NRORG'];
        $this->cdfilial   = $row['CDFILIAL'];
        $this->nrConsPlan = $row['NRCONSPLAN'];
        /* 
                Rotina era utilizada quando Curva ABC era exibida no grid e não via BIRT
        
        $result = array();
        switch($row['REPORT_NAME']){
            case 'Cus01400':
                $result = $this->curvaGeralPlanejado($row, $codigo);
            break;
            case 'Cus01401':
                $result = $this->curvaGeralRecalculo($row, $codigo);
                break;
        }*/
    }

    public function curvaGeralPlanejado ($row, $codigo) {
        $this->dataSourceName = 'cus01400_abcgeralplan';
        $this->parameterBagColumns = array('P_NRCONSPLAN', 'P_CDFILIAL', 'P_NRTIPOCUSTO', 'P_DTMESINIC', 'P_DTMESFINA', 'P_CDPRODINIC', 'P_CDPRODFINA', 'P_IDNESTOCAVEIS');
        
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('P_NRCONSPLAN',       $codigo);
        $filterCriteria->addCondition('P_CDFILIAL',         $row['CDFILIAL']);
        $filterCriteria->addCondition('P_NRTIPOCUSTO',      $row['CDTIPOCUSTO']);
        // $filterCriteria->addCondition('P_TIPODATA',         $row['TPDTCUS']);
        $filterCriteria->addCondition('P_DTMESINIC',        $row['DTMESINIC']);
        $filterCriteria->addCondition('P_DTMESFINA',        $row['DTMESFINA']);
        $filterCriteria->addCondition('P_CDPRODINIC',       $row['CDPRODUTOINI']);
        $filterCriteria->addCondition('P_CDPRODFINA',       $row['CDPRODUTOFINAL']);
        $filterCriteria->addCondition('P_IDNESTOCAVEIS',    $row['IDNESTOCAVEIS']);        

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        return $dataSet;
    }
    
    public function curvaGeralRecalculo ($row, $codigo) {
        $this->dataSourceName = 'cus01400_abcgeralrecalc';
        $this->parameterBagColumns = array('P_NRCONSPLAN', 'P_CDFILIAL', 'P_NMCUSTO', 'P_IDCONSFIXO', 'P_NRTIPOCUSTO', 'P_DTMESINIC', 'P_DTMESFINA', 'P_DTFINAL', 'P_DTINICIAL', 'P_DTPERIFINA', 'P_PRODZER', 'P_IDMATEPRIM', 'P_IDCONSPERC', 'P_CDPRODUTOINIC', 'P_CDPRODUTOFINA', 'P_CDPRODINIC', 'P_CDPRODFINA', 'P_IDNESTOCAVEIS');
        
        $filterCriteria = new FilterCriteria($this->dataSourceName);
        $filterCriteria->addCondition('P_NRCONSPLAN',       $codigo);
        $filterCriteria->addCondition('P_CDFILIAL',         $row['CDFILIAL']);
        $filterCriteria->addCondition('P_NMCUSTO',          $row['NMCUSTO']);
        $filterCriteria->addCondition('P_IDCONSFIXO',       $row['IDCONSFIXO']);
        $filterCriteria->addCondition('P_NRTIPOCUSTO',      $row['CDTIPOCUSTO']);
        $filterCriteria->addCondition('P_DTMESINIC',        $row['DTMESINIC']);
        $filterCriteria->addCondition('P_DTMESFINA',        $row['DTMESFINA']);
        $filterCriteria->addCondition('P_DTFINAL',          $row['DTMESFINA']);
        $filterCriteria->addCondition('P_DTINICIAL',        $row['DTMESINIC']);
        $filterCriteria->addCondition('P_DTPERIFINA',       $row['DTMESFINA']);
        $filterCriteria->addCondition('P_PRODZER',          $row['PRODZER']);
        $filterCriteria->addCondition('P_IDMATEPRIM',       $row['IDMATEPRIM']);
        $filterCriteria->addCondition('P_IDCONSPERC',       $row['IDCONSPERC']);
        $filterCriteria->addCondition('P_CDPRODUTOINIC',    $row['CDPRODUTOINI']);
        $filterCriteria->addCondition('P_CDPRODUTOFINA',    $row['CDPRODUTOFINAL']);
        $filterCriteria->addCondition('P_CDPRODINIC',       $row['CDARVPRODINI']);
        $filterCriteria->addCondition('P_CDPRODFINA',       $row['CDARVPRODFINAL']);
        $filterCriteria->addCondition('P_IDNESTOCAVEIS',    $row['IDNESTOCAVEIS']);        

        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);

        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        return $dataSet;
    }

    public function excluiConsplanCus01400(DTO\Request\Row $request, DTO\Response $response) {
        $parametros = array ("NRORG" => $this->environment->getNrOrg(), "CDFILICPLA" =>$this->environment->getCdFilial(), "NRCONSPLAN" => $this->codigo, "__is_new" => false);
        $newRequest = new DTO\Request\Row($parametros, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        $newResponse = new DTO\Response();
        $this->consplan->delete($newRequest, $newResponse);
    }
}